﻿using WingRpcSDK.Common;
using WingRpcSDK.JsonRpc.Driver;
using WingRpcSDK.Log;
using System;
using System.Diagnostics;

namespace WingCloudServer
{
    internal class InProcessServiceLoader : BaseServiceLoader
    {
        private DllWatcher _dllWatcher;

        /// <summary>
        /// RegisterService
        /// </summary>
        /// <param name="serviceType"></param>
        /// <param name="service"></param>
        internal override void RegisterService(Type serviceType, object service)
        {
            var baseService = (BaseRpcService)service;
            baseService.InProcessClient = RpcInProcessClient;
            baseService.Name = string.IsNullOrWhiteSpace(baseService.Name) ? serviceType.Name.Trim('I') : baseService.Name;
            CallRegister(RpcInProcessService, serviceType, service);
            baseService.InitService();
        }

        /// <summary>
        /// Start service
        /// </summary>
        internal override void StartService()
        {
            //RpcInProcessService.Start();
        }

        /// <summary>
        /// Stop service
        /// </summary>
        internal override void StopService()
        {
            RpcInProcessService.End();
        }

        internal override void LoadDlls(string path)
        {
            _dllWatcher = new DllWatcher(path, () => {
                lock(RestartLocker)
                {
                    FileLoggerSingle.Instance.WriteWarn("InProcessServiceLoader", "LoadDlls", "0", "Server is reboot now...");
                    Process.Start("ServerReboot").WaitForExit();
                }
            });
            base.LoadDlls(path);
        }
    }
}
