﻿using WingRpcSDK.Common;
using WingRpcSDK.JsonRpc.Driver;
using WingRpcSDK.Log;
using JsonRpcLite.Network;
using System;
using System.Collections.Concurrent;
using System.Diagnostics;

namespace WingCloudServer
{
    internal class NetServiceLoader: BaseServiceLoader
    {
        private DllWatcher _dllWatcher;
        private RpcServer _netRpcService;
        private ConcurrentDictionary<string, BaseRpcService> _netRpcServiceDic = new();

        public NetServiceLoader(int port)
        {
            _netRpcService = new(new JsonRpcHttpServerEngine($"http://*:{port}/"));
        }

        /// <summary>
        /// Register service
        /// </summary>
        /// <param name="serviceType">Service type</param>
        /// <param name="service">Service instance</param>
        internal override void RegisterService(Type serviceType, object service)
        {
            var baseService = (BaseRpcService)service;
            baseService.InProcessClient = RpcInProcessClient;
            baseService.InitService();
            baseService.Name = string.IsNullOrWhiteSpace(baseService.Name) ? serviceType.Name.Trim('I') : baseService.Name;
            CallRegister(_netRpcService, serviceType, service);
            _netRpcServiceDic.AddOrUpdate(baseService.Name, baseService, (k, v) => baseService);
        }


        /// <summary>
        /// Start service
        /// </summary>
        internal override void StartService()
        {
            _netRpcService?.Start();
        }

        /// <summary>
        /// Stop service
        /// </summary>
        internal override void StopService()
        {
            _netRpcService?.End();
        }

        internal override void LoadDlls(string path)
        {
            _dllWatcher = new DllWatcher(path, () => {
                lock (RestartLocker)
                {
                    FileLoggerSingle.Instance.WriteWarn("NetServiceLoader","LoadDlls","0","Server is reboot now...");
                    Process.Start("ServerReboot").WaitForExit();
                }
            });
            base.LoadDlls(path);
        }
    }
}
