﻿using WingRpcSDK.Common;
using WingRpcSDK.Log;
using FluentScheduler;
using Microsoft.Extensions.Configuration;
using System;
using System.Diagnostics;
using System.Threading.Tasks;
using System.IO;

namespace WingCloudServer
{
    class Program
    {
        static void Main(string[] args)
        {
            while (!StartServer())
            {
                Console.WriteLine("The server is abnormal,wait 5 second will to reboot server!");
                Task.Delay(5000).Wait();
            }
        }

        private static bool StartServer()
        {
            try
            {
                JobManager.Initialize();
                var builder = new ConfigurationBuilder().SetBasePath(AppDomain.CurrentDomain.BaseDirectory).AddJsonFile("appsettings.json");
                var configurationRoot = builder.Build();
                var rpcServerPort = configurationRoot.GetSection("RpcServerPort");
                var staticServerPort = configurationRoot.GetSection("StaticServerPort");
                int.TryParse(rpcServerPort.Value, out int rpcServerPortValue);
                IServiceContainer rpcServiceContainer = new ThreadServiceContainer(rpcServerPortValue);
                rpcServiceContainer.LoadDlls();
                rpcServiceContainer.StartService();
                if (Process.GetProcessesByName("Sms.Tool").Length == 0)
                {
                    Process.Start(Path.Combine(AppDomain.CurrentDomain.BaseDirectory,"Sms.Tool.exe"));
                }
                if (Process.GetProcessesByName("GoStaticSev").Length == 0)
                {
                    Process.Start(Path.Combine(AppDomain.CurrentDomain.BaseDirectory,"GoStaticSev"), $" -port={staticServerPort.Value} -path=Storage");
                }
                Console.WriteLine($"{rpcServiceContainer.ServiceName} Running!");
                Console.ReadLine();
                rpcServiceContainer.StopService();
                return true;
            }
            catch (Exception ex)
            {
                FileLoggerSingle.Instance.WriteError("Program", "Main", "0", $"Applaction exception {ex} exit,reboot now!");
                return false;
            }
        }
    }
}
